@@ -35,7 +35,7 @@
 
 #include "videodev2.h"
 
-#include "ivtv.h"
+#include "media/ivtv.h"
 
 static void detect(int fh, struct v4l2_sliced_vbi_format *fmt)
 {
