@@ -36,14 +36,8 @@
 
 #include "videodev2.h"
 
-#include "ivtv.h"
+#include "media/ivtv.h"
 
-struct v4l2_register {
-	__u32 i2c_id; 		/* I2C driver ID of the I2C chip. 0 for the I2C adapter. */
-	unsigned long reg;
-	__u32 val;
-};
-#define	VIDIOC_INT_G_REGISTER 		_IOWR('d', 101, struct v4l2_register)
 #define I2C_DRIVERID_CX25840	71	/* cx2584x video encoder	*/
 #define I2C_DRIVERID_SAA711X	73	/* saa711x video encoders	*/
 #define I2C_DRIVERID_SAA717X	80	/* saa717x video encoder	*/
@@ -542,19 +536,20 @@ static void cc_readback(int fh)
 	int cc_cnt = 0;
 	int field = -1;
 
-	v.i2c_id = I2C_DRIVERID_CX25840;
+	v.match_type = V4L2_CHIP_MATCH_I2C_DRIVER;
+	v.match_chip = I2C_DRIVERID_CX25840;
 	v.reg = 0x444;
-	if (ioctl(fh, VIDIOC_INT_G_REGISTER, &v))
-		v.i2c_id = I2C_DRIVERID_SAA711X;
+	if (ioctl(fh, VIDIOC_DBG_G_REGISTER, &v))
+		v.match_chip = I2C_DRIVERID_SAA711X;
 
 	while (1) {
-		if (v.i2c_id == I2C_DRIVERID_CX25840) {
+		if (v.match_chip == I2C_DRIVERID_CX25840) {
 			v.reg = 0x444;
-			ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+			ioctl(fh, VIDIOC_DBG_G_REGISTER, &v);
 			if ((v.val & 32) == 32) {
 				int skip = v.val;
 				v.reg = 0x445;
-				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				ioctl(fh, VIDIOC_DBG_G_REGISTER, &v);
 				if (skip & 16)
 					continue;
 				parity[cc_cnt] = v.val & 128;
@@ -562,7 +557,7 @@ static void cc_readback(int fh)
 				if (cc_cnt == 2) {
 					data.field = 0;
 					if (odd_parity(data.data[0] &&
-								odd_parity(data.data[1])))
+							odd_parity(data.data[1])))
 						decode_cc(&data);
 					cc_cnt = 0;
 				}
@@ -571,13 +566,13 @@ static void cc_readback(int fh)
 		}
 		else {
 			v.reg = 0x66;
-			ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+			ioctl(fh, VIDIOC_DBG_G_REGISTER, &v);
 			if ((v.val & 0x30) == 0 && (v.val & 0xe) != field) {
 				v.reg = 0x69;
-				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				ioctl(fh, VIDIOC_DBG_G_REGISTER, &v);
 				data.data[0] = v.val;
 				v.reg = 0x6a;
-				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				ioctl(fh, VIDIOC_DBG_G_REGISTER, &v);
 				data.data[1] = v.val;
 				data.field = 0;
 				if (odd_parity(data.data[0] && odd_parity(data.data[1])))
