@@ -35,10 +35,22 @@
 #include <sys/time.h>
 #include <math.h>
 #include <linux/types.h>
-#include <linux/version.h>
 #include <sys/klog.h>
 
 #include "videodev2.h"
+#include "video.h"
+
+/* copied from ivtv-driver.h */
+#define IVTV_DBGFLG_WARN  (1 << 0)
+#define IVTV_DBGFLG_INFO  (1 << 1)
+#define IVTV_DBGFLG_API   (1 << 2)
+#define IVTV_DBGFLG_DMA   (1 << 3)
+#define IVTV_DBGFLG_IOCTL (1 << 4)
+#define IVTV_DBGFLG_I2C   (1 << 5)
+#define IVTV_DBGFLG_IRQ   (1 << 6)
+#define IVTV_DBGFLG_DEC   (1 << 7)
+#define IVTV_DBGFLG_YUV   (1 << 8)
+#define IVTV_DBGFLG_HIGHVOL   (1 << 9)
 
 /* Internals copied from media/v4l2-common.h */
 struct v4l2_routing {
@@ -47,36 +59,9 @@ struct v4l2_routing {
 };
 #define	VIDIOC_INT_S_AUDIO_ROUTING	_IOW ('d', 109, struct v4l2_routing)
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
-#define V4L2_CHIP_MATCH_HOST       0  /* Match against chip ID on host (0 for the host) */
-#define V4L2_CHIP_MATCH_I2C_DRIVER 1  /* Match against I2C driver ID */
-#define V4L2_CHIP_MATCH_I2C_ADDR   2  /* Match against I2C 7-bit address */
-
-struct v4l2_register {
-	__u32 match_type; /* Match type */
-	__u32 i2c_id; /* Match this chip, meaning determined by match_type */
-	__u64 reg;
-	__u64 val;
-};
-#define	VIDIOC_INT_S_REGISTER 	_IOW  ('V', 79, struct v4l2_register)
-#define	VIDIOC_INT_G_REGISTER 	_IOWR ('V', 80, struct v4l2_register)
-#else
-struct v4l2_register {
-	__u32 i2c_id; 		/* I2C driver ID of the I2C chip. 0 for the I2C adapter. */
-	unsigned long reg;
-	__u32 val;
-};
-#define	VIDIOC_INT_S_REGISTER 		_IOW ('d', 100, struct v4l2_register)
-#define	VIDIOC_INT_G_REGISTER 		_IOWR('d', 101, struct v4l2_register)
-#endif
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
 #define VIDIOC_INT_RESET            	_IOW ('d', 102, __u32)
-#else
-#define VIDIOC_INT_RESET            	_IO  ('d', 102)
-#endif
 
-#define IVTV_INTERNAL
-#include "ivtv.h"
+#include "media/ivtv.h"
 
 #define __stringify_1(x)	#x
 #define __stringify(x)		__stringify_1(x)
@@ -92,10 +77,7 @@ struct v4l2_register {
 enum Option {
 	OptSetDebugLevel = 'D',
 	OptSetDevice = 'd',
-	OptEndGop = 'E',
 	OptGetDebugLevel = 'e',
-	OptListRegisters = 'R',
-	OptSetRegister = 'r',
 	OptReloadFW = 'H',
 	OptHelp = 'h',
 	OptSetGPIO = 'i',
@@ -103,8 +85,6 @@ enum Option {
 	OptPassThrough = 'K',
 	OptFrameSync = 'k',
 	OptSetAudioRoute = 'v',
-	OptGetVbiPassThrough = 'W',
-	OptSetVbiPassThrough = 'w',
 	OptLast = 128
 };
 
@@ -116,17 +96,13 @@ static int option_getYuvMode = 0;
 
 /* YUV mode specified */
 #define YUVMode 		(1L<<0)
-#define YUVThreshold		(1L<<1)
 
 static struct option long_options[] = {
 	/* Please keep in alphabetical order of the short option.
 	   That makes it easier to see which options are still free. */
 	{"set-debug", required_argument, 0, OptSetDebugLevel},
 	{"device", required_argument, 0, OptSetDevice},
-	{"end-gop", required_argument, 0, OptEndGop},
 	{"get-debug", no_argument, 0, OptGetDebugLevel},
-	{"list-registers", required_argument, 0, OptListRegisters},
-	{"set-register", required_argument, 0, OptSetRegister},
 	{"reload", no_argument, 0, OptReloadFW},
 	{"help", no_argument, 0, OptHelp},
 	{"set-gpio", required_argument, 0, OptSetGPIO},
@@ -134,8 +110,6 @@ static struct option long_options[] = {
 	{"passthrough", required_argument, 0, OptPassThrough},
 	{"sync", no_argument, 0, OptFrameSync},
 	{"audio-route", required_argument, 0, OptSetAudioRoute},
-	{"get-vbi-passthrough", no_argument, 0, OptGetVbiPassThrough},
-	{"set-vbi-passthrough", required_argument, 0, OptSetVbiPassThrough},
 	{"reset-ir", no_argument, &option_reset_ir, 1},
 	{"version", no_argument, &option_version, 1},
 	{"get-yuv-mode", no_argument, &option_getYuvMode, 1},
@@ -151,11 +125,12 @@ static void usage(void)
 	printf("  -K, --passthrough <mode>\n");
 	printf("                     set passthrough mode: 1 = on, 0 = off [IVTV_IOC_PASSTHROUGH]\n");
 	printf("  --get-yuv-mode     display the current yuv mode\n");
-	printf("  --set-yuv-mode [param],[param]\n");
-        printf("                     set the current yuv mode\n");
-	printf("     param:\n");
-	printf("       mode        =<#> yuv_mode 0=interlaced,1=progressive,2=auto\n");
-	printf("       threshold   =<#> if src_hight <= threshold treat as progressive\n");
+	printf("  --set-yuv-mode mode=<mode>\n");
+        printf("                     set the current yuv mode:\n");
+	printf("       		     mode 0: interlaced (top transmitted first)\n");
+	printf("       		     mode 1: interlaced (bottom transmitted first)\n");
+	printf("       		     mode 2: progressive\n");
+	printf("       		     mode 3: auto\n");
 	printf("  --reset-ir         reset the infrared receiver [VIDIOC_INT_RESET]\n");
 	printf("  --version          shows the version number of this utility.\n");
         printf("                     It should match the driver version.\n");
@@ -171,23 +146,16 @@ static void usage(void)
 	printf("                     show GPIO input/direction/output bits\n");
 	printf("  -i, --set-gpio [dir=<dir>,]val=<val>\n");
 	printf("                     set GPIO direction bits to <dir> and set output to <val>\n");
-	printf("  -k, --sync         test vsync's capabilities [IVTV_IOC_FRAMESYNC]\n");
-	printf("  -R, --list-registers chip=<chip>,min=<addr>,max=<addr>\n");
-	printf("                     dump registers from <min> to <max> [VIDIOC_INT_G_REGISTER]\n");
-	printf("                     <chip> is one of: saa711x, saa7127, saa717x, cx2584x, cx2341x\n");
-	printf("  -r, --set-register chip=<chip>,reg=<reg>,val=<val>\n");
-	printf("                     set register [VIDIOC_INT_S_REGISTER]\n");
-	printf("                     <chip> is one of: saa711x, saa7127, saa717x, cx2584x, cx2341x\n");
+	printf("  -k, --sync         test vsync's capabilities [VIDEO_GET_EVENT]\n");
 	printf("  -v, --audio-route=input=<in>,output=<out>\n");
 	printf("                     set the audio input/output routing [VIDIOC_INT_S_AUDIO_ROUTING]\n");
 	exit(0);
 }
 
-static char *pts_to_string(char *str, unsigned long pts)
+static char *pts_to_string(char *str, unsigned long pts, float fps)
 {
 	static char buf[256];
 	int hours, minutes, seconds, fracsec;
-	float fps;
 	int frame;
 	char *p = (str) ? str : buf;
 
@@ -201,11 +169,9 @@ static char *pts_to_string(char *str, unsigned long pts)
 	hours = minutes / 60;
 	minutes = minutes % 60;
 
-	fps = 30;
 	frame = (int)ceilf(((float)fracsec / (float)MPEG_CLOCK_FREQ) * fps);
 
-	snprintf(p, sizeof(buf), "%d:%02d:%02d:%d", hours, minutes, seconds,
-		 frame);
+	snprintf(p, sizeof(buf), "%d:%02d:%02d:%d", hours, minutes, seconds, frame);
 	return p;
 }
 
@@ -286,6 +252,34 @@ static char *doread(const char *fn)
 	return buf;
 }
 
+static const char *field2s(int val)
+{
+	switch (val) {
+	case V4L2_FIELD_ANY:
+		return "Any";
+	case V4L2_FIELD_NONE:
+		return "None";
+	case V4L2_FIELD_TOP:
+		return "Top";
+	case V4L2_FIELD_BOTTOM:
+		return "Bottom";
+	case V4L2_FIELD_INTERLACED:
+		return "Interlaced";
+	case V4L2_FIELD_SEQ_TB:
+		return "Sequential Top-Bottom";
+	case V4L2_FIELD_SEQ_BT:
+		return "Sequential Bottom-Top";
+	case V4L2_FIELD_ALTERNATE:
+		return "Alternating";
+	case V4L2_FIELD_INTERLACED_TB:
+		return "Interlaced Top-Bottom";
+	case V4L2_FIELD_INTERLACED_BT:
+		return "Interlaced Bottom-Top";
+	default:
+		return "Unknown";
+	}
+}
+
 static int doioctl(int fd, int request, void *parm, const char *name)
 {
 	int retVal;
@@ -300,86 +294,21 @@ static int doioctl(int fd, int request, void *parm, const char *name)
 	return retVal;
 }
 
-/* Copied from <linux/i2c-id.h> */
-#define I2C_DRIVERID_SAA711X	73	/* saa711x video encoders	*/
-#define I2C_DRIVERID_SAA717X	80	/* saa717x video encoder	*/
-#define I2C_DRIVERID_SAA7127	72	/* saa7124 video encoder	*/
-#define I2C_DRIVERID_CX25840	71	/* cx2584x video encoder	*/
-
-static int parse_chip(const char *chip)
-{
-	if (!strcmp(chip, "saa711x"))
-		return I2C_DRIVERID_SAA711X;
-	if (!strcmp(chip, "saa717x"))
-		return I2C_DRIVERID_SAA717X;
-	if (!strcmp(chip, "saa7127"))
-		return I2C_DRIVERID_SAA7127;
-	if (!strcmp(chip, "cx2584x"))
-		return I2C_DRIVERID_CX25840;
-	if (!strcmp(chip, "cx2341x"))
-		return 0;
-	return -1;
-}
-
-static void print_regs(int fd, struct v4l2_register *reg, unsigned long min, unsigned long max, int stride)
-{
-	unsigned long mask = stride > 1 ? 0x1f : 0x0f;
-	unsigned long i;
-	int line = 0;
-
-	for (i = min & ~mask; i <= max; i += stride) {
-		if ((i & mask) == 0 && line % 32 == 0) {
-			if (stride == 4)
-				printf("\n                00       04       08       0C       10       14       18       1C");
-			else
-				printf("\n          00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
-		}
-	
-		if ((i & mask) == 0) {
-			printf("\n%08lx: ", i);
-			line++;
-		}
-		if (i < min) {
-			printf("%*s ", 2 * stride, "");
-			continue;
-		}
-		reg->reg = i;
-		if (ioctl(fd, VIDIOC_INT_G_REGISTER, reg) < 0) {
-			fprintf(stderr, "ioctl: VIDIOC_INT_G_REGISTER "
-					"failed for 0x%08x\n", (__u32)reg->reg);
-		} else {
-			printf("%0*x ", 2 * stride, (__u32)reg->val);
-		}
-		usleep(1);
-	}
-	printf("\n");
-}
-
 int main(int argc, char **argv)
 {
 	char *value, *subs;
 	int i;
 	char *subopts[] = {
-#define SUB_REG				0
-		"reg",
-#define SUB_VAL				1
+#define SUB_VAL				0
 		"val",
-#define SUB_MAX				2
-		"max",
-#define SUB_MIN				3
-		"min",
-#define SUB_YUV_MODE			4
+#define SUB_YUV_MODE			1
 		"mode",
-#define SUB_YUV_THRESHOLD		5
-		"threshold",
-#define SUB_DIR				6
+#define SUB_DIR				2
 		"dir",
-#define SUB_INPUT                       7
+#define SUB_INPUT                       3
 		"input",
-#define SUB_OUTPUT                      8
+#define SUB_OUTPUT                      4
 		"output",
-#define SUB_CHIP			9
-		"chip",
 
 		NULL
 	};
@@ -392,28 +321,18 @@ int main(int argc, char **argv)
 	/* command args */
 	char *device = strdup("/dev/video0");	/* -d device */
         int ch;
-	struct ivtv_ioctl_yuv_interlace yuv_mode = { 0, 0 };
-	struct v4l2_register reg;
-	unsigned long min_reg = 0;	/* min registers to list */
-	unsigned long max_reg = 0;	/* max registers to list: 0 == undefined */
+	int yuv_mode = 0;
 	struct v4l2_routing route;	/* audio_route */
 	unsigned short gpio_out = 0x0;	/* GPIO output data */
 	unsigned short gpio_dir = 0x0;	/* GPIO direction bits */
 	int gpio_set_dir = 0;
 	int passthrough = 0;
-	int vbi_passthrough = 0;
 	int debug_level = 0;
-	struct ivtv_ioctl_framesync frameinfo;
-	int new_debug_level, gdebug_level, end_gop = 0;
+	int new_debug_level, gdebug_level;
 	double timestamp;
 	char ptsstr[64];
-	char scrstr[64];
 	char short_options[26 * 2 * 2 + 1];
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
-	reg.match_type = V4L2_CHIP_MATCH_I2C_DRIVER;
-#endif
-
 	if (argc == 1) {
 		usage();
 		return 0;
@@ -451,18 +370,12 @@ int main(int argc, char **argv)
 						return 1;
 
 					}
-					yuv_mode.interlace_mode = strtol(value, 0L, 0);
-					set_yuvMode |= YUVMode;
-					break;
-				case SUB_YUV_THRESHOLD:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <threshold>\n");
-						usage();
-						exit(1);
+					yuv_mode = strtol(value, 0L, 0);
+					if (yuv_mode < 0 || yuv_mode > 3) {
+						fprintf(stderr, "invalid yuv mode\n");
+						return 1;
 					}
-					yuv_mode.threshold = strtol(value, 0L, 0);
-					set_yuvMode |= YUVThreshold;
+					set_yuvMode |= YUVMode;
 					break;
                                 }
                         }
@@ -471,10 +384,6 @@ int main(int argc, char **argv)
 		case OptHelp:
 			usage();
 			return 0;
-		case OptEndGop:{
-			end_gop = strtol(optarg, 0L, 0);
-			break;
-		}
 		case OptSetDebugLevel:{
 			debug_level = strtol(optarg, 0L, 0);
 			break;
@@ -487,118 +396,6 @@ int main(int argc, char **argv)
                                 sprintf(device, "/dev/video%c", dev);
                         }
 			break;
-
-		case OptSetVbiPassThrough:
-			if (strstr(optarg, "cc"))
-				vbi_passthrough |= V4L2_SLICED_CAPTION_525;
-			if (strstr(optarg, "wss"))
-				vbi_passthrough |= V4L2_SLICED_WSS_625;
-			if (strstr(optarg, "vps"))
-				vbi_passthrough |= V4L2_SLICED_VPS;
-			if (strstr(optarg, "none"))
-				vbi_passthrough = 0;
-			break;
-
-		case OptSetRegister:
-			subs = optarg;
-			while (*subs != '\0') {
-				switch (getsubopt(&subs, subopts, &value)) {
-				case SUB_CHIP:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <chip>\n");
-						usage();
-						exit(1);
-					}
-					reg.i2c_id = parse_chip(value);
-					if (reg.i2c_id < 0) {
-						fprintf(stderr,
-							"Unknown chip specification\n");
-						usage();
-						exit(1);
-					}
-					break;
-				case SUB_REG:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <reg>\n");
-						usage();
-						exit(1);
-					}
-					reg.reg =
-					    (unsigned long)strtoul(value, 0UL,
-								   0);
-					break;
-				case SUB_VAL:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <val>\n");
-						usage();
-						exit(1);
-					}
-					reg.val =
-					    (unsigned long)strtoul(value, 0UL,
-								   0);
-					break;
-				default:
-					fprintf(stderr,
-						"Invalid suboptions specified\n");
-					usage();
-					exit(1);
-					break;
-				}
-			}
-			break;
-		case OptListRegisters:
-			subs = optarg;
-			while (*subs != '\0') {
-				switch (getsubopt(&subs, subopts, &value)) {
-				case SUB_CHIP:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <chip>\n");
-						usage();
-						exit(1);
-					}
-					reg.i2c_id = parse_chip(value);
-					if (reg.i2c_id < 0) {
-						fprintf(stderr,
-							"Unknown chip specification\n");
-						usage();
-						exit(1);
-					}
-					break;
-				case SUB_MIN:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <reg>\n");
-						usage();
-						exit(1);
-					}
-					min_reg = (unsigned long)
-					    strtol(value, 0L, 0);
-					break;
-				case SUB_MAX:
-					if (value == NULL) {
-						fprintf(stderr,
-							"No value given to suboption <reg>\n");
-						usage();
-						exit(1);
-					}
-					max_reg = (unsigned long)
-					    strtol(value, 0L, 0);
-					break;
-				default:
-					break;
-				}
-			}
-			if (min_reg > max_reg) {
-				fprintf(stderr, "Invalid suboptions specified\n");
-				usage();
-				exit(1);
-				break;
-			}
-			break;
 		case OptSetAudioRoute:
 			subs = optarg;
 			while (*subs != '\0') {
@@ -693,88 +490,36 @@ int main(int argc, char **argv)
 
 	/* Setting Opts */
 
-	if (options[OptEndGop]) {
-		if (doioctl(fd, IVTV_IOC_S_GOP_END, &end_gop,
-			"IVTV_IOC_S_GOP_END") == 0)
-			printf("End GOP capture set to %d\n", end_gop);
-	}
-
-	if (options[OptSetRegister]) {
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
-		if (reg.i2c_id == 0) {
-			reg.match_type = V4L2_CHIP_MATCH_HOST;
-		}
-#endif
-		if (doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
-			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("register 0x%08x set to 0x%08x\n",
-		       		(__u32)reg.reg, (__u32)reg.val);
-	}
-
 	if (options[OptSetAudioRoute]) {
 		doioctl(fd, VIDIOC_INT_S_AUDIO_ROUTING, &route, "VIDIOC_INT_S_AUDIO_ROUTING");
 	}
 
-	if (options[OptListRegisters]) {
-		int stride = 1;
-
-		if (reg.i2c_id == 0) {
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
-			reg.match_type = V4L2_CHIP_MATCH_HOST;
-#endif
-			stride = 4;
-		}
-		printf("ioctl: VIDIOC_INT_G_REGISTER\n");
-		if (max_reg == 0) {
-			switch (reg.i2c_id) {
-			case I2C_DRIVERID_SAA711X:
-				print_regs(fd, &reg, 0, 0xff, stride);
-				break;
-			case I2C_DRIVERID_SAA717X:
-				// FIXME: use correct reg regions
-				print_regs(fd, &reg, 0, 0xff, stride);
-				break;
-			case I2C_DRIVERID_SAA7127:
-				print_regs(fd, &reg, 0, 0x7f, stride);
-				break;
-			case I2C_DRIVERID_CX25840:
-				print_regs(fd, &reg, 0, 2, stride);
-				print_regs(fd, &reg, 0x100, 0x15f, stride);
-				print_regs(fd, &reg, 0x200, 0x23f, stride);
-				print_regs(fd, &reg, 0x400, 0x4bf, stride);
-				print_regs(fd, &reg, 0x800, 0x9af, stride);
-				break;
-			case 0:
-				print_regs(fd, &reg, 0x02000000, 0x0200ffff, stride);
-				break;
-			}
-		}
-		else {
-			print_regs(fd, &reg, min_reg, max_reg, stride);
-		}
-	}
-
 	if (options[OptFrameSync]) {
-		printf("ioctl: IVTV_IOC_FRAMESYNC\n");
+		printf("ioctl: VIDEO_GET_EVENT\n");
 
 		for (;;) {
-			if (ioctl(fd, IVTV_IOC_FRAMESYNC, &frameinfo) < 0) {
-				fprintf(stderr,
-					"ioctl: IVTV_IOC_FRAMESYNC failed\n");
+			struct video_event ev;
+			int fps = 30;
+			v4l2_std_id std;
+
+			if (ioctl(fd, VIDIOC_G_STD, &std) == 0)
+				fps = (std & V4L2_STD_525_60) ? 30 : 25;
+			if (ioctl(fd, VIDEO_GET_EVENT, &ev) < 0) {
+				fprintf(stderr, "ioctl: VIDEO_GET_EVENT failed\n");
 				break;
-			} else {
+			} else if (ev.timestamp == 0) {
+				unsigned long long pts = 0, frame = 0;
 				struct timeval tv;
 				gettimeofday(&tv, NULL);
 				timestamp =
 				    (double)tv.tv_sec +
 				    ((double)tv.tv_usec / 1000000.0);
 
-				pts_to_string(ptsstr, frameinfo.pts);
-				pts_to_string(scrstr, frameinfo.scr);
-				printf
-				    ("%10.6f: pts %-20s, scr %-20s, %d frames\n",
-				     timestamp, ptsstr, scrstr,
-				     frameinfo.frame);
+				ioctl(fd, VIDEO_GET_PTS, &pts);
+				ioctl(fd, VIDEO_GET_FRAME_COUNT, &frame);
+				pts_to_string(ptsstr, pts, fps);
+				printf("%10.6f: pts %-20s, %lld frames\n",
+				     timestamp, ptsstr, frame);
 			}
 		}
 	}
@@ -782,38 +527,34 @@ int main(int argc, char **argv)
 	if (options[OptSetGPIO]) {
 		struct v4l2_register reg;
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
 		reg.match_type = V4L2_CHIP_MATCH_HOST;
-#endif
-		reg.i2c_id = 0;
+		reg.match_chip = 0;
 		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
 		reg.val = gpio_dir;
-		if (gpio_set_dir && doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
-			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("GPIO dir set to 0x%04x\n", (__u32)reg.val);
+		if (gpio_set_dir && doioctl(fd, VIDIOC_DBG_S_REGISTER, &reg,
+			"VIDIOC_DBG_S_REGISTER") == 0)
+			printf("GPIO dir set to 0x%04llx\n", reg.val);
 		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
 		reg.val = gpio_out;
-		if (doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
-			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("GPIO out set to 0x%04x\n", (__u32)reg.val);
+		if (doioctl(fd, VIDIOC_DBG_S_REGISTER, &reg,
+			"VIDIOC_DBG_S_REGISTER") == 0)
+			printf("GPIO out set to 0x%04llx\n", reg.val);
 	}
 
 	if (options[OptListGPIO]) {
 		struct v4l2_register reg;
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
 		reg.match_type = V4L2_CHIP_MATCH_HOST;
-#endif
-		reg.i2c_id = 0;
+		reg.match_chip = 0;
 		reg.reg = IVTV_REG_GPIO_IN_OFFSET;
-		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO in:  0x%04x\n", (__u32)reg.val);
+		if (ioctl(fd, VIDIOC_DBG_G_REGISTER, &reg) == 0) 
+			printf("GPIO in:  0x%04llx\n", reg.val);
 		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
-		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO dir: 0x%04x\n", (__u32)reg.val);
+		if (ioctl(fd, VIDIOC_DBG_G_REGISTER, &reg) == 0) 
+			printf("GPIO dir: 0x%04llx\n", reg.val);
 		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
-		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO out: 0x%04x\n", (__u32)reg.val);
+		if (ioctl(fd, VIDIOC_DBG_G_REGISTER, &reg) == 0) 
+			printf("GPIO out: 0x%04llx\n", reg.val);
 	}
 
 	if (options[OptSetDebugLevel]) {
@@ -841,16 +582,13 @@ int main(int argc, char **argv)
 	}
 
 	if (options[OptPassThrough]) {
-		doioctl(fd, IVTV_IOC_PASSTHROUGH, &passthrough, "IVTV_IOC_PASSTHROUGH");
+		doioctl(fd, VIDEO_SELECT_SOURCE, (void *)(unsigned long)(passthrough ? VIDEO_SOURCE_DEMUX : VIDEO_SOURCE_MEMORY), "IVTV_IOC_PASSTHROUGH");
 	}
 
 	if (option_reset_ir) {
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
 		__u32 dummy = 0;
+
 		doioctl(fd, VIDIOC_INT_RESET, &dummy, "VIDIOC_INT_RESET");
-#else
-		doioctl(fd, VIDIOC_INT_RESET, NULL, "VIDIOC_INT_RESET");
-#endif
 	}
 
 	if (option_version) {
@@ -859,38 +597,40 @@ int main(int argc, char **argv)
 
         if (option_setYuvMode)
         {
-            printf("set yuv mode\n");
-            struct ivtv_ioctl_yuv_interlace interlace_mode;
-            doioctl(fd, IVTV_IOC_G_YUV_INTERLACE, &interlace_mode, "IVTV_IOC_G_YUV_INTERLACE");
-            if (set_yuvMode & YUVMode)
-                interlace_mode.interlace_mode = yuv_mode.interlace_mode;
-            if (set_yuvMode & YUVThreshold)
-                interlace_mode.threshold = yuv_mode.threshold;
-            doioctl(fd, IVTV_IOC_S_YUV_INTERLACE, &interlace_mode, "IVTV_IOC_S_YUV_INTERLACE");
+		struct ivtv_dma_frame frame;
+		struct v4l2_format fmt;
+		const enum v4l2_field map[4] = {
+			V4L2_FIELD_INTERLACED_TB,
+			V4L2_FIELD_INTERLACED_BT,
+			V4L2_FIELD_NONE,
+			V4L2_FIELD_ANY,
+		};
 
+		printf("set yuv mode\n");
+		memset(&frame, 0, sizeof(frame));
+		if (ioctl(fd, IVTV_IOC_DMA_FRAME, &frame) < 0) {
+			fprintf(stderr, "Unable to switch to user DMA YUV mode\n");
+			exit(1);
+		}
+		fmt.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
+		ioctl(fd, VIDIOC_G_FMT, &fmt);
+		fmt.fmt.pix.field = map[yuv_mode];
+		doioctl(fd, VIDIOC_S_FMT, &fmt, "VIDIOC_S_FMT");
         }
 
         if (option_getYuvMode)
         {
-            struct ivtv_ioctl_yuv_interlace interlace_mode;
-            doioctl(fd, IVTV_IOC_G_YUV_INTERLACE, &interlace_mode, "IVTV_IOC_G_YUV_INTERLACE");
-            printf("Current yuv_mode %d ",interlace_mode.interlace_mode);
-            switch(interlace_mode.interlace_mode)
-            {
-            case IVTV_YUV_MODE_INTERLACED:
-                printf("interlaced\n");
-                break;
-            case IVTV_YUV_MODE_PROGRESSIVE:
-                printf("progressive\n");
-                break;
-            case IVTV_YUV_MODE_AUTO:
-                printf("auto\n");
-                break;
-            default:
-                printf("undefined\n");
-                break;
-            }
-            printf("\tauto switch threshold %d\n",interlace_mode.threshold);
+		struct ivtv_dma_frame frame;
+		struct v4l2_format fmt;
+		
+		memset(&frame, 0, sizeof(frame));
+		if (ioctl(fd, IVTV_IOC_DMA_FRAME, &frame) < 0) {
+			fprintf(stderr, "Unable to switch to user DMA YUV mode\n");
+			exit(1);
+		}
+		fmt.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
+		doioctl(fd, VIDIOC_G_FMT, &fmt, "VIDIOC_G_FMT");
+		printf("Current yuv_mode %d %s\n", fmt.fmt.pix.field, field2s(fmt.fmt.pix.field));
         }
 
 	close(fd);
