@@ -117,6 +117,7 @@ enum Option {
 	OptGetOutputCropCap,
 	OptGetOverlayCropCap,
 	OptGetOutputOverlayCropCap,
+	OptOverlay,
 	OptLast = 256
 };
 
@@ -226,6 +227,7 @@ static struct option long_options[] = {
 	{"get-cropcap-output-overlay", no_argument, 0, OptGetOutputOverlayCropCap},
 	{"get-crop-output-overlay", no_argument, 0, OptGetOutputOverlayCrop},
 	{"set-crop-output-overlay", required_argument, 0, OptSetOutputOverlayCrop},
+	{"overlay", required_argument, 0, OptOverlay},
 	{0, 0, 0, 0}
 };
 
@@ -266,10 +268,6 @@ static void usage(void)
 	printf("  -O, --get-output   query the video output [VIDIOC_G_OUTPUT]\n");
 	printf("  -o, --set-output=<num>\n");
 	printf("                     set the video output to <num> [VIDIOC_S_OUTPUT]\n");
-	printf("  -Q, --list-audio-outputs\n");
-	printf("                     display audio outputs [VIDIOC_ENUMAUDOUT]\n");
-	printf("  -q, --list-audio-inputs\n");
-	printf("                     display audio inputs [VIDIOC_ENUMAUDIO]\n");
 	printf("  -S, --get-standard\n");
 	printf("                     query the video standard [VIDIOC_G_STD]\n");
 	printf("  -s, --set-standard=<num>\n");
@@ -317,9 +315,9 @@ static void usage(void)
 	printf("                     vps:      VPS (PAL/SECAM)\n");
 	printf("  --get-fmt-vbi      query the VBI capture format [VIDIOC_G_FMT]\n");
 	printf("  --get-fmt-vbi-out  query the VBI output format [VIDIOC_G_FMT]\n");
+	printf("  --overlay=<on>     turn overlay on (1) or off (0) (VIDIOC_OVERLAY)\n");
 	printf("  --get-fbuf         query the overlay framebuffer data [VIDIOC_G_FBUF]\n");
-	printf("  --set-fbuf=chromakey=<0/1>,global_alpha=<0/1>,local_alpha=<0/1>,\n");
-	printf("             global_inv_alpha=<0/1>,local_inv_alpha=<0/1>\n");
+	printf("  --set-fbuf=chromakey=<0/1>,global_alpha=<0/1>,local_alpha=<0/1>,local_inv_alpha=<0/1>\n");
 	printf("		     set the overlay framebuffer [VIDIOC_S_FBUF]\n");
 	printf("  --get-cropcap      query the crop capabilities [VIDIOC_CROPCAP]\n");
 	printf("  --get-crop	     query the video capture crop window [VIDIOC_G_CROP]\n");
@@ -650,8 +648,6 @@ static std::string fbufcap2s(unsigned cap)
 		s += "\t\t\tGlobal Alpha\n";
 	if (cap & V4L2_FBUF_CAP_LOCAL_ALPHA)
 		s += "\t\t\tLocal Alpha\n";
-	if (cap & V4L2_FBUF_CAP_GLOBAL_INV_ALPHA)
-		s += "\t\t\tGlobal Inverted Alpha\n";
 	if (cap & V4L2_FBUF_CAP_LOCAL_INV_ALPHA)
 		s += "\t\t\tLocal Inverted Alpha\n";
 	if (cap & V4L2_FBUF_CAP_LIST_CLIPPING)
@@ -676,8 +672,6 @@ static std::string fbufflags2s(unsigned fl)
 		s += "\t\t\tGlobal Alpha\n";
 	if (fl & V4L2_FBUF_FLAG_LOCAL_ALPHA)
 		s += "\t\t\tLocal Alpha\n";
-	if (fl & V4L2_FBUF_FLAG_GLOBAL_INV_ALPHA)
-		s += "\t\t\tGlobal Inverted Alpha\n";
 	if (fl & V4L2_FBUF_FLAG_LOCAL_INV_ALPHA)
 		s += "\t\t\tLocal Inverted Alpha\n";
 	if (s.empty()) s += "\t\t\t\n";
@@ -1171,6 +1165,7 @@ int main(int argc, char **argv)
 	double freq = 0;		/* get/set frequency */
 	struct v4l2_frequency vf;	/* get_freq/set_freq */
 	struct v4l2_standard vs;	/* list_std */
+	int overlay;			/* overlay */
 	char short_options[26 * 2 * 2 + 1];
 	int idx = 0;
 
@@ -1298,7 +1293,6 @@ int main(int argc, char **argv)
 					"chromakey",
 					"global_alpha",
 					"local_alpha",
-					"global_inv_alpha",
 					"local_inv_alpha",
 					NULL
 				};
@@ -1317,16 +1311,15 @@ int main(int argc, char **argv)
 					set_fbuf |= V4L2_FBUF_FLAG_LOCAL_ALPHA;
 					break;
 				case 3:
-					fbuf.flags |= strtol(value, 0L, 0) ? V4L2_FBUF_FLAG_GLOBAL_INV_ALPHA : 0;
-					set_fbuf |= V4L2_FBUF_FLAG_GLOBAL_INV_ALPHA;
-					break;
-				case 4:
 					fbuf.flags |= strtol(value, 0L, 0) ? V4L2_FBUF_FLAG_LOCAL_INV_ALPHA : 0;
 					set_fbuf |= V4L2_FBUF_FLAG_LOCAL_INV_ALPHA;
 					break;
 				}
 			}
 			break;
+		case OptOverlay:
+			overlay = strtol(optarg, 0L, 0);
+			break;
 		case OptSetCrop:
 			parse_crop(optarg, set_crop, vcrop);
 			break;
@@ -1702,6 +1695,10 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (options[OptOverlay]) {
+		doioctl(fd, VIDIOC_OVERLAY, &overlay, "VIDIOC_OVERLAY");
+	}
+
 	if (options[OptSetCrop]) {
 		do_crop(fd, set_crop, vcrop, V4L2_BUF_TYPE_VIDEO_CAPTURE);
 	}
